package com.abcinc.phonelist;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * Implementation of <strong>Action</strong> that displays a list of
 * phone contacts and gives the user the ability to edit a phone
 * contact, create a new phone contact, or delete one or more phone
 * contacts.
 *
 * @author Wellie W. Chao
 */
public class ShowListAction extends Action {

	// ------------------------------------------------------- Static Variables

	private static boolean isInitialized = false;

	// ----------------------------------------------------- Instance Variables

	// ----------------------------------------------------------- Constructors

	// -------------------------------------------------------- Private Methods

	// ------------------------------------------------- Private Static Methods

	// ----------------------------------------------------- Accessors/Mutators

	// --------------------------------------------------------- Public Methods

	/**
	 * Process the specified HTTP request, and create the corresponding HTTP
	 * response (or forward to another web component that will create it).
	 * Return an <code>ActionForward</code> instance describing where and how
	 * control should be forwarded, or <code>null</code> if the response has
	 * already been completed.
	 *
	 * @param mapping The ActionMapping used to select this instance
	 * @param form The optional ActionForm bean for this request (if any)
	 * @param request The HTTP request we are processing
	 * @param response The HTTP response we are creating
	 *
	 * @exception Exception if the application business logic throws an
	 * exception
	 */
	public ActionForward execute
		(ActionMapping mapping,
		 ActionForm form,
		 HttpServletRequest request,
		 HttpServletResponse response)
		throws Exception {

		if (!isInitialized) {
			initSampleData();
			isInitialized = true;
		}

		// get the list of contacts in memory
		List contactBeanList = ContactDatabase.getList();

		// put displayed information in request attribute context for the JSP
		request.setAttribute("contactBeanList", contactBeanList);

		// Forward control to the success page
		return (mapping.findForward("success"));

	}

	// -------------------------------------------------- Public Static Methods

	/**
	 * Populate the database with some sample data. The caller should
	 * make sure initSampleData is only called once per
	 * deployment. Otherwise, the database will quickly fill up with
	 * repeated requests to this Action.
	 */
	private void initSampleData() {
		ContactBean contactBean = null;

		contactBean = new ContactBean();
		contactBean.setId(Sequence.getId());
		contactBean.setLastName("Chao");
		contactBean.setFirstName("Wellie");
		contactBean.setPhone("212-655-5412");
		contactBean.setFax("866-788-8109");
		contactBean.setEmail("wchao@yahoo.com");
		ContactDatabase.addBean(contactBean);

		contactBean = new ContactBean();
		contactBean.setId(Sequence.getId());
		contactBean.setLastName("Feferman");
		contactBean.setFirstName("Michael");
		contactBean.setPhone("505-385-0318");
		contactBean.setFax("505-555-1313");
		contactBean.setEmail("ownshow@yahoo.com");
		ContactDatabase.addBean(contactBean);

		contactBean = new ContactBean();
		contactBean.setId(Sequence.getId());
		contactBean.setLastName("Weld");
		contactBean.setFirstName("David");
		contactBean.setPhone("650-804-2040");
		contactBean.setFax("");
		contactBean.setEmail("dweld@stanford.edu");
		ContactDatabase.addBean(contactBean);

		contactBean = new ContactBean();
		contactBean.setId(Sequence.getId());
		contactBean.setLastName("Lee");
		contactBean.setFirstName("Troy");
		contactBean.setPhone("213-891-7567");
		contactBean.setFax("");
		contactBean.setEmail("tslee98@yahoo.com");
		ContactDatabase.addBean(contactBean);

		contactBean = new ContactBean();
		contactBean.setId(Sequence.getId());
		contactBean.setLastName("Lee");
		contactBean.setFirstName("Jennifer");
		contactBean.setPhone("202-862-0394");
		contactBean.setFax("");
		contactBean.setEmail("jenny@nytimes.com");
		ContactDatabase.addBean(contactBean);

		contactBean = new ContactBean();
		contactBean.setId(Sequence.getId());
		contactBean.setLastName("Overholt");
		contactBean.setFirstName("Alison");
		contactBean.setPhone("650-616-4135");
		contactBean.setFax("");
		contactBean.setEmail("alison@oulifen.com");
		ContactDatabase.addBean(contactBean);
	}

}
