package com.abcinc.phonelist.test;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.SubmitButton;
import com.meterware.httpunit.TableCell;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebLink;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;
import java.util.Random;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Test case fixture for the edit function of the phone list
 * application. This fixture contains several tests, each of which is
 * expressed in a method with a name of the form testXXX.
 *
 * @author Wellie Chao
 */
public class EditTest extends TestCase {

	// configurable constants.
	private static String protocol = "http";
	private static String hostname = "localhost";
	private static int port = 8080;
	private static String showListPath = "/phonelist/showList.do";
	private static String newPath = "/phonelist/new.do";
	private static String savePath = "/phonelist/save.do";
	private static String randomCharacterSet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

	// variables
	private static Random random = new Random();
	private WebConversation webConversation = new WebConversation();

	// main method so class can run from command line.
	public static void main(String args[]) {
		junit.textui.TestRunner.run(suite());
	}

	public static TestSuite suite() {
		return new TestSuite(EditTest.class);
	}

	public EditTest(String s) {
		super(s);
	}

	// test that editing a contact causes changes to be saved properly.
	public void testEditContact() throws Exception {
		String url;
		WebRequest webRequest;
		WebResponse webResponse;
		WebTable webTable;
		WebForm webForm;
		String lastName;
		String firstName;
		String phone;
		String fax;
		String email;
		SubmitButton saveButton;
		int rowCount;
		int numContacts;
		TableCell lastBodyCell;
		// first create a new contact so that we can later edit it.
		url = protocol + "://" + hostname + ":" + port + newPath;
		webRequest = new GetMethodWebRequest(url);
		webResponse = webConversation.getResponse(webRequest);
		webForm = webResponse.getFormWithID("contactForm");
		lastName = getRandomString(8);
		firstName = getRandomString(8);
		phone = getRandomString(8);
		fax = getRandomString(8);
		email = getRandomString(8);
		webForm.setParameter("lastName", lastName);
		webForm.setParameter("firstName", firstName);
		webForm.setParameter("phone", phone);
		webForm.setParameter("fax", fax);
		webForm.setParameter("email", email);
		// find the save button (rather than the cancel button).
		saveButton = webForm.getSubmitButton("submitButton", "	 Save		");
		webResponse = webForm.submit(saveButton);
		// now parse the results after saving the contact.
		webTable = webResponse.getTableStartingWith("Phone List");
		rowCount = webTable.getRowCount();
		// subtract the three rows for title, column headings, and buttons.
		numContacts = rowCount - 3;
		lastBodyCell = webTable.getTableCell(rowCount - 2, 0);
		if (lastBodyCell.getColSpan() > 1) {
			// this means the message that there were no contacts was
			// displayed.
			numContacts = 0;
		}
		// find the contact we just created in the list of contacts.
		Integer matchingContactId = null;
		WebLink matchingWebLink = null;
		for(int i = 0; i < numContacts; i++) {
			// first two rows are table title and heading columns
			int rowIndex = i + 2;
			// get the last name
			TableCell tableCell = webTable.getTableCell(rowIndex, 2);
			String columnValue = tableCell.asText();
			if (columnValue.equals(lastName)) {
				tableCell = webTable.getTableCell(rowIndex, 3);
				columnValue = tableCell.asText();
				if (!columnValue.equals(firstName)) continue;
				tableCell = webTable.getTableCell(rowIndex, 4);
				columnValue = tableCell.asText();
				if (!columnValue.equals(phone)) continue;
				tableCell = webTable.getTableCell(rowIndex, 5);
				columnValue = tableCell.asText();
				if (!columnValue.equals(fax)) continue;
				// email has an extra space at the beginning and an extra
				// space at the end, so we need to strip those.
				tableCell = webTable.getTableCell(rowIndex, 6);
				columnValue = tableCell.asText();
				columnValue = columnValue.substring(1, columnValue.length() - 1);
				if (!columnValue.equals(email)) continue;
				// get the ID value of the contact, stored in the "Edit" link
				// in the second column.
				tableCell = webTable.getTableCell(rowIndex, 1);
				matchingWebLink = tableCell.getLinkWith("Edit");
				String[] idParameterArray = matchingWebLink.getParameterValues("id");
				matchingContactId = new Integer(idParameterArray[0]);
				break;
			}
		}
		// pretend we clicked on the edit link.
		webResponse = matchingWebLink.click();
		webForm = webResponse.getFormWithID("contactForm");
		// test that the contact form exists.
		assertNotNull("Contact form not found", webForm);
		// test that the required fields exist on the form.
		assertTrue("Last Name field (lastName) missing from contact form", webForm.hasParameterNamed("lastName"));
		assertTrue("First Name field (firstName) missing from contact form", webForm.hasParameterNamed("firstName"));
		assertTrue("Phone field (phone) missing from contact form", webForm.hasParameterNamed("phone"));
		assertTrue("Fax field (fax) missing from contact form", webForm.hasParameterNamed("fax"));
		assertTrue("Email field (email) missing from contact form", webForm.hasParameterNamed("email"));
		// test that the form action points to the save action.
		String formAction = webForm.getAction();
		// strip off session state information in the URL if it is there
		// because we want to compare the path of the URL.
		if (formAction.indexOf(";jsessionid=") >= 0) {
			formAction = formAction.substring(0, formAction.indexOf(";jsessionid="));
		}
		assertTrue("Contact form points to wrong action", formAction.endsWith(savePath));
		lastName = getRandomString(8);
		firstName = getRandomString(8);
		phone = getRandomString(8);
		fax = getRandomString(8);
		email = getRandomString(8);
		webForm.setParameter("lastName", lastName);
		webForm.setParameter("firstName", firstName);
		webForm.setParameter("phone", phone);
		webForm.setParameter("fax", fax);
		webForm.setParameter("email", email);
		// find the save button (rather than the cancel button).
		saveButton = webForm.getSubmitButton("submitButton", "	 Save		");
		webResponse = webForm.submit(saveButton);
		// now parse the results after saving the changed contact.
		webTable = webResponse.getTableStartingWith("Phone List");
		rowCount = webTable.getRowCount();
		// subtract the three rows for title, column headings, and buttons.
		numContacts = rowCount - 3;
		lastBodyCell = webTable.getTableCell(rowCount - 2, 0);
		if (lastBodyCell.getColSpan() > 1) {
			// this means the message that there were no contacts was
			// displayed.
			numContacts = 0;
		}
		// compare the contact to make sure our edits took effect.
		boolean foundMatchingContact = false;
		for(int i = 0; i < numContacts; i++) {
			// first two rows are table title and heading columns
			int rowIndex = i + 2;
			// get the ID value of the contact, stored in the "Edit" link
			// in the second column.
			TableCell tableCell = webTable.getTableCell(rowIndex, 1);
			WebLink webLink = tableCell.getLinkWith("Edit");
			String[] idParameterArray = webLink.getParameterValues("id");
			Integer id = new Integer(idParameterArray[0]);
			if (matchingContactId.equals(id)) {
				tableCell = webTable.getTableCell(rowIndex, 2);
				String columnValue = tableCell.asText();
				if (!columnValue.equals(lastName)) continue;
				tableCell = webTable.getTableCell(rowIndex, 3);
				columnValue = tableCell.asText();
				if (!columnValue.equals(firstName)) continue;
				tableCell = webTable.getTableCell(rowIndex, 4);
				columnValue = tableCell.asText();
				if (!columnValue.equals(phone)) continue;
				tableCell = webTable.getTableCell(rowIndex, 5);
				columnValue = tableCell.asText();
				if (!columnValue.equals(fax)) continue;
				// email has an extra space at the beginning and an extra
				// space at the end, so we need to strip those.
				tableCell = webTable.getTableCell(rowIndex, 6);
				columnValue = tableCell.asText();
				columnValue = columnValue.substring(1, columnValue.length() - 1);
				if (!columnValue.equals(email)) continue;
				foundMatchingContact = true;
				break;
			}
		}
		assertTrue("Edit did not successfully change a contact.", foundMatchingContact);
		// test that the cancel button works.
		webResponse = matchingWebLink.click();
		webForm = webResponse.getFormWithID("contactForm");
		String newLastName = getRandomString(8);
		String newFirstName = getRandomString(8);
		String newPhone = getRandomString(8);
		String newFax = getRandomString(8);
		String newEmail = getRandomString(8);
		webForm.setParameter("lastName", newLastName);
		webForm.setParameter("firstName", newFirstName);
		webForm.setParameter("phone", newPhone);
		webForm.setParameter("fax", newFax);
		webForm.setParameter("email", newEmail);
		SubmitButton[] submitButtonArray = webForm.getSubmitButtons();
		SubmitButton cancelButton = null;
		for(int i = 0; i < submitButtonArray.length; i++) {
			SubmitButton submitButton = submitButtonArray[i];
			if (submitButton.getValue().equals("Cancel")) {
				cancelButton = submitButton;
			}
		}
		assertNotNull("Missing cancel button", cancelButton);
		webResponse = webForm.submit(cancelButton);
		// now parse the results after clicking on cancel.
		webTable = webResponse.getTableStartingWith("Phone List");
		rowCount = webTable.getRowCount();
		// subtract the three rows for title, column headings, and buttons.
		numContacts = rowCount - 3;
		lastBodyCell = webTable.getTableCell(rowCount - 2, 0);
		if (lastBodyCell.getColSpan() > 1) {
			// this means the message that there were no contacts was
			// displayed.
			numContacts = 0;
		}
		// compare the contact to make sure that our edits did NOT take
		// effect if we clicked on cancel. In other words, make sure the
		// contact retains the old values we set them to a few lines
		// above.
		foundMatchingContact = false;
		for(int i = 0; i < numContacts; i++) {
			// first two rows are table title and heading columns
			int rowIndex = i + 2;
			// get the ID value of the contact, stored in the "Edit" link
			// in the second column.
			TableCell tableCell = webTable.getTableCell(rowIndex, 1);
			WebLink webLink = tableCell.getLinkWith("Edit");
			String[] idParameterArray = webLink.getParameterValues("id");
			Integer id = new Integer(idParameterArray[0]);
			if (matchingContactId.equals(id)) {
				tableCell = webTable.getTableCell(rowIndex, 2);
				String columnValue = tableCell.asText();
				if (!columnValue.equals(lastName)) continue;
				tableCell = webTable.getTableCell(rowIndex, 3);
				columnValue = tableCell.asText();
				if (!columnValue.equals(firstName)) continue;
				tableCell = webTable.getTableCell(rowIndex, 4);
				columnValue = tableCell.asText();
				if (!columnValue.equals(phone)) continue;
				tableCell = webTable.getTableCell(rowIndex, 5);
				columnValue = tableCell.asText();
				if (!columnValue.equals(fax)) continue;
				// email has an extra space at the beginning and an extra
				// space at the end, so we need to strip those.
				tableCell = webTable.getTableCell(rowIndex, 6);
				columnValue = tableCell.asText();
				columnValue = columnValue.substring(1, columnValue.length() - 1);
				if (!columnValue.equals(email)) continue;
				foundMatchingContact = true;
				break;
			}
		}
		assertTrue("Cancel button on edit form makes changes rather than discarding changes", foundMatchingContact);
		// now delete the newly created contact.
		webForm = webResponse.getFormWithID("contactListForm");
		// choose the contact we just created.
		String[] idParameterArray = new String[1];
		idParameterArray[0] = matchingContactId.toString();
		webForm.setParameter("id", idParameterArray);
		SubmitButton deleteButton = webForm.getSubmitButton("submitButton", "Delete Selected");
		// submit the form with the two contacts selected.
		webResponse = webForm.submit(deleteButton);
	}

	// generate a random string of the specified length.
	private String getRandomString(int length) {
		StringBuffer randomStringBuffer = new StringBuffer();
		for(int i = 0; i < length; i++) {
			int index = random.nextInt(randomCharacterSet.length());
			randomStringBuffer.append(randomCharacterSet.substring(index, index + 1));
		}
		return randomStringBuffer.toString();
	}

}
